#include <stdlib.h>
#include <GL/gl.h> 
#include <math.h> 
#include "glrnd.h" 

#define INITIAL_NUMBER_OF_POINTS 1000

double* Color(double f, double *rgbx)
{
  double per=6.28;
  rgbx[0]=0.5*(cos(per*f)+1.0);
  rgbx[1]=0.5*(cos(per*f-2.0*3.141/3.0)+1.0);
  rgbx[2]=0.5*(cos(per*f-4.0*3.141/3.0)+1.0);
  rgbx[0]=f;
  rgbx[1]=0;
  rgbx[2]=1-f;
  
  return rgbx;
}

typedef double Coord[3];
Coord *Points;
static int quit_application=0; 
static int number_of_points=0;
static void draw(glRenderer rnd, void * user_data)
{ 
  /*Cast data to actual type. */
  Coord* points=(Coord*)user_data;   
  double color[4];
  int i;
  glShadeModel(GL_SMOOTH);
  glPointSize(2);
  glBegin(GL_LINE_STRIP);
  for(i=0;i<number_of_points;i++)
  {  
    glColor3dv(Color(((double)i)/((double)(number_of_points)),color));
    glVertex3dv(points[i]);
  }
  glEnd();
}

int  do_quit_application(glRenderer rnd, int kbd_mod_mask)
{ 
  quit_application=1; 
  return GL_FALSE;    
}

int  main(int argc, char * argv[])
{ 
  glRenderer rnd;
  int i,j,id,ialloc;
  
  double min[3],max[3];
  for (id=0;id<3;id++) min[id]=1.0e30;
  for (id=0;id<3;id++) max[id]=-1.0e30;
  ialloc=INITIAL_NUMBER_OF_POINTS;
  Points=malloc(sizeof(Coord)*ialloc);
  i=0;
  while (!feof(stdin))
    {
      fscanf(stdin,"%le %le %le\n",&Points[i][0],&Points[i][1],&Points[i][2]);
      for (id=0;id<3;id++)
	{
	  if (Points[i][id]>max[id]) max[id]=Points[i][id];
	  if (Points[i][id]<min[id]) min[id]=Points[i][id];
	}
      i++;
      if (i>=ialloc)
	{
	  printf("realloc\n");
	  ialloc=1.5*i;
	  Points=realloc(Points,sizeof(Coord)*ialloc);
	}
    }
  number_of_points=i;
  
  printf("%d points read\n",i);
    printf("volume: [%e:%e]x[%e:%e]x[%e:%e] \n",min[0],max[0],min[1],max[1],min[2],max[2]);
  rnd=glrCreate("Trajectory view",0,0,500,500);
  
  /* Set the 3D volume. */
  glrSetVolume(rnd,min[0],max[0],min[1],max[1],min[2],max[2]);

  
  /* Register key action callback for quitting the appliacation. */
  glrRegisterKeyAction(rnd,GLW_q,do_quit_application,"q:Quit");
  
  /* Initialize random number generator and calculate
     the initial positions of the points. */

  /* Put renderer into application controlled (non-dialog) mode. */
  glrSetDialog(rnd,1);
  
  glRender(rnd,draw,Points);
      
  /* Free the memory.*/
  glrDestroy(rnd);
  free(Points);
  return 0;
}
